@@ -10,5 +10,5 @@ const platformName = "aws-ecs"
 
 // Options are the SDK options to use for instantiation.
 var Options = []sdk.Option{
-	sdk.WithComponents(&Platform{}),
+	sdk.WithComponents(&Platform{}, &TaskLauncher{}),
 }
