@@ -6,6 +6,7 @@ import (
 	"github.com/zclconf/go-cty/cty/gocty"
 
 	"github.com/hashicorp/waypoint/internal/config/funcs"
+	"github.com/hashicorp/waypoint/internal/config/variables"
 )
 
 // EvalContext returns the common eval context to use for parsing all
@@ -95,6 +96,13 @@ func finalizeContext(ctx *hcl.EvalContext) *hcl.EvalContext {
 	return ctx
 }
 
+// AddVariables uses the final map of InputValues to add all input variables
+// to the given hcl EvalContext.
+func AddVariables(ctx *hcl.EvalContext, vs variables.Values) *hcl.EvalContext {
+	variables.AddInputVariables(ctx, vs)
+	return ctx
+}
+
 // hclContextContainer is an interface that config structs that have an HCL
 // context may implement. We use this for certain things such as mapoperation()
 // to set the proper context.
