@@ -1,6 +1,7 @@
 package variables
 
 import (
+	"context"
 	"os"
 	"path/filepath"
 	"testing"
@@ -12,10 +13,13 @@ import (
 	"github.com/hashicorp/hcl/v2/gohcl"
 	"github.com/hashicorp/hcl/v2/hclsimple"
 	"github.com/hashicorp/hcl/v2/hclsyntax"
-	pb "github.com/hashicorp/waypoint/internal/server/gen"
 	"github.com/stretchr/testify/require"
 	"github.com/zclconf/go-cty/cty"
 	"github.com/zclconf/go-cty/cty/convert"
+
+	"github.com/hashicorp/waypoint/internal/appconfig"
+	"github.com/hashicorp/waypoint/internal/plugin"
+	pb "github.com/hashicorp/waypoint/internal/server/gen"
 )
 
 func TestVariables_DecodeVariableBlock(t *testing.T) {
@@ -35,6 +39,10 @@ func TestVariables_DecodeVariableBlock(t *testing.T) {
 			"invalid_def.hcl",
 			"Invalid default value",
 		},
+		{
+			"invalid_type_dynamic.hcl",
+			"must be string",
+		},
 	}
 
 	for _, tt := range cases {
@@ -55,7 +63,7 @@ func TestVariables_DecodeVariableBlock(t *testing.T) {
 			for _, block := range content.Blocks {
 				switch block.Type {
 				case "variable":
-					v, decodeDiag := decodeVariableBlock(block)
+					v, decodeDiag := decodeVariableBlock(nil, block)
 					vs[block.Labels[0]] = v
 					if decodeDiag.HasErrors() {
 						diags = append(diags, decodeDiag...)
@@ -176,6 +184,103 @@ func TestVariables_LoadVCSFile(t *testing.T) {
 	}
 }
 
+func TestVariables_LoadDynamicDefaults(t *testing.T) {
+	cases := []struct {
+		name     string
+		file     string
+		provided []*pb.Variable
+		needs    bool
+		expected map[string]string
+		err      string
+	}{
+		{
+			"no dynamic",
+			"no_dynamic.hcl",
+			nil,
+			false,
+			nil,
+			"",
+		},
+
+		{
+			"dynamic but provided",
+			"dynamic.hcl",
+			[]*pb.Variable{
+				{
+					Name: "teeth",
+					Value: &pb.Variable_Str{
+						Str: "pointy",
+					},
+				},
+			},
+			false,
+			nil,
+			"",
+		},
+
+		{
+			"dynamic need value",
+			"dynamic.hcl",
+			[]*pb.Variable{
+				{
+					Name: "irrelevent",
+					Value: &pb.Variable_Str{
+						Str: "NO",
+					},
+				},
+			},
+			true,
+			map[string]string{
+				"teeth": "hello",
+			},
+			"",
+		},
+	}
+	for _, tt := range cases {
+		t.Run(tt.name, func(t *testing.T) {
+			require := require.New(t)
+
+			file := filepath.Join("testdata", tt.file)
+			base := testConfig{}
+
+			err := hclsimple.DecodeFile(file, nil, &base)
+			require.NoError(err)
+
+			schema, _ := gohcl.ImpliedBodySchema(&testConfig{})
+			content, diags := base.Body.Content(schema)
+			require.False(diags.HasErrors())
+
+			vars, diags := DecodeVariableBlocks(nil, content)
+			require.False(diags.HasErrors(), diags.Error())
+
+			needs := NeedsDynamicDefaults(tt.provided, vars)
+			require.Equal(tt.needs, needs)
+
+			dynVars, diags := LoadDynamicDefaults(
+				context.Background(),
+				hclog.L(),
+				tt.provided,
+				vars,
+				appconfig.WithPlugins(map[string]*plugin.Instance{
+					"static": {
+						Component: &appconfig.StaticConfigSourcer{},
+					},
+				}),
+			)
+			require.False(diags.HasErrors())
+
+			actual := map[string]string{}
+			for _, v := range dynVars {
+				actual[v.Name] = v.Value.(*pb.Variable_Str).Str
+			}
+			if len(actual) == 0 {
+				actual = nil
+			}
+			require.Equal(tt.expected, actual)
+		})
+	}
+}
+
 func TestVariables_EvalInputValues(t *testing.T) {
 	cases := []struct {
 		name        string
@@ -193,11 +298,19 @@ func TestVariables_EvalInputValues(t *testing.T) {
 					Value:  &pb.Variable_Str{Str: "gdbee"},
 					Source: &pb.Variable_Cli{},
 				},
+				{
+					Name:   "dynamic",
+					Value:  &pb.Variable_Str{Str: "value"},
+					Source: &pb.Variable_Cli{},
+				},
 			},
 			expected: Values{
 				"art": &Value{
 					cty.StringVal("gdbee"), "cli", hcl.Expression(nil), hcl.Range{},
 				},
+				"dynamic": &Value{
+					cty.StringVal("value"), "cli", hcl.Expression(nil), hcl.Range{},
+				},
 				"is_good": &Value{
 					cty.BoolVal(false), "default", hcl.Expression(nil), hcl.Range{},
 				},
@@ -317,7 +430,7 @@ func TestVariables_EvalInputValues(t *testing.T) {
 			for _, block := range content.Blocks {
 				switch block.Type {
 				case "variable":
-					v, decodeDiag := decodeVariableBlock(block)
+					v, decodeDiag := decodeVariableBlock(nil, block)
 					vs[block.Labels[0]] = v
 					if decodeDiag.HasErrors() {
 						diags = append(diags, decodeDiag...)
@@ -326,14 +439,18 @@ func TestVariables_EvalInputValues(t *testing.T) {
 			}
 			require.False(diags.HasErrors())
 
-			ivs, diags := EvaluateVariables(tt.inputValues, vs, hclog.New(&hclog.LoggerOptions{}))
+			ivs, diags := EvaluateVariables(
+				hclog.New(&hclog.LoggerOptions{}),
+				tt.inputValues,
+				vs,
+			)
 			if tt.err != "" {
 				require.True(diags.HasErrors())
 				require.Contains(diags.Error(), tt.err)
 				return
 			}
 
-			require.False(diags.HasErrors())
+			require.False(diags.HasErrors(), diags.Error())
 			for k, v := range tt.expected {
 				diff := cmp.Diff(v, ivs[k], cmpOpts...)
 				if diff != "" {
