@@ -18,6 +18,7 @@ import (
 	"github.com/hashicorp/waypoint-plugin-sdk/component"
 	"github.com/hashicorp/waypoint-plugin-sdk/datadir"
 	"github.com/hashicorp/waypoint-plugin-sdk/terminal"
+	"github.com/hashicorp/waypoint/internal/appconfig"
 	configpkg "github.com/hashicorp/waypoint/internal/config"
 	"github.com/hashicorp/waypoint/internal/config/variables"
 	"github.com/hashicorp/waypoint/internal/core"
@@ -54,16 +55,37 @@ func (r *Runner) executeJob(
 	// update our job later.
 	var configInfo pb.Job_Config
 
-	// Eventually we'll need to extract the data source. For now we're
-	// just building for local exec so it is the working directory.
+	// Try to load the waypoint.hcl from the working directory first.
 	configInfo.Source = pb.Job_Config_FILE
 	path, err := configpkg.FindPath(wd, "", false)
 	if err != nil {
 		return nil, err
 	}
+
+	// Waypoint.hcl set on the job overrides all, even if we found a
+	// waypoint.hcl in the working directory above.
+	if job.WaypointHcl != nil && len(job.WaypointHcl.Contents) > 0 {
+		log.Info("using waypoint.hcl associated with the project in the server")
+
+		// ext has the extra extension information for the file. We add
+		// ".json" if this is JSON-formatted.
+		ext := ""
+		if job.WaypointHcl.Format == pb.Hcl_JSON {
+			ext = ".json"
+		}
+
+		// We just write this into the working directory.
+		path = filepath.Join(wd, configpkg.Filename+ext)
+		if err := ioutil.WriteFile(path, job.WaypointHcl.Contents, 0644); err != nil {
+			return nil, status.Errorf(codes.Internal,
+				"Failed to write waypoint.hcl from job metadata: %s", err)
+		}
+
+		configInfo.Source = pb.Job_Config_JOB
+	}
+
+	// If we still have no path, try to load from the project.
 	if path == "" {
-		// If no waypoint.hcl file is found in the downloaded data, look for
-		// a default waypoint HCL.
 		log.Trace("waypoint.hcl not found in downloaded data, looking for default in server")
 		if v := resp.Project.WaypointHcl; len(v) > 0 {
 			log.Info("using waypoint.hcl associated with the project in the server")
@@ -71,7 +93,7 @@ func (r *Runner) executeJob(
 			// ext has the extra extension information for the file. We add
 			// ".json" if this is JSON-formatted.
 			ext := ""
-			if resp.Project.WaypointHclFormat == pb.Project_JSON {
+			if resp.Project.WaypointHclFormat == pb.Hcl_JSON {
 				ext = ".json"
 			}
 
@@ -160,13 +182,51 @@ func (r *Runner) executeJob(
 		return nil, diags
 	}
 
+	// Combine all our known variables into a list of variables. This
+	// determines the precedence order of values.
 	pbVars := resp.Project.GetVariables()
 	pbVars = append(pbVars, envVars...)
 	pbVars = append(pbVars, vcsVars...)
 	pbVars = append(pbVars, job.Variables...)
 
+	// Load any dynamic default values. This happens after the above
+	// because we only load dynamic default values for variables we do
+	// not have values for.
+	if variables.NeedsDynamicDefaults(pbVars, cfg.InputVariables) {
+		// If we're a local runner, we can't support this. We don't
+		// support dynamic config sourcing on local runners since it
+		// requires config sourcing plugins, auth, etc.
+		if r.local {
+			return nil, status.Errorf(
+				codes.FailedPrecondition,
+				"Variables with dynamic defaults cannot be used with local "+
+					"runners. Projects using variables with dynamic defaults must "+
+					"be executed remotely using a remote runner.",
+			)
+		}
+
+		dynamicVars, diags := variables.LoadDynamicDefaults(
+			ctx,
+			log,
+			pbVars,
+			cfg.InputVariables,
+			appconfig.WithLogger(log),
+			appconfig.WithPlugins(r.configPlugins),
+		)
+		if diags.HasErrors() {
+			return nil, diags
+		}
+		if len(dynamicVars) > 0 {
+			// If we have dynamic variable values, we _prepend_ them so that
+			// they have the lowest precedence. In reality, this shouldn't
+			// matter because we only grab dynamic values for vars that have
+			// no value set, but we might as well be careful.
+			pbVars = append(dynamicVars, pbVars...)
+		}
+	}
+
 	// evaluate all variables against the variable blocks we just decoded
-	inputVars, diags := variables.EvaluateVariables(pbVars, cfg.InputVariables, log)
+	inputVars, diags := variables.EvaluateVariables(log, pbVars, cfg.InputVariables)
 	if diags.HasErrors() {
 		return nil, diags
 	}
