@@ -11,7 +11,7 @@ import (
 	"google.golang.org/grpc/status"
 
 	pb "github.com/hashicorp/waypoint/internal/server/gen"
-	"github.com/hashicorp/waypoint/internal/server/singleprocess/state"
+	"github.com/hashicorp/waypoint/internal/serverstate"
 )
 
 // applicationPoll accepts a state management interface which provides access
@@ -21,7 +21,7 @@ import (
 type applicationPoll struct {
 	// state is the state management interface that provides functions for
 	// safely mutating server state.
-	state *state.State
+	state serverstate.Interface
 
 	// the workspace to check for polling applications and running their status on
 	// Currently, the application poll handler is only expected to work for default
