@@ -149,6 +149,9 @@ func TestServiceGetJobStream_complete(t *testing.T) {
 		require.NotNil(assignment)
 		require.Equal(queueResp.JobId, assignment.Assignment.Job.Id)
 
+		// The assigned runner ID has been set for the job
+		require.Equal(id, assignment.Assignment.Job.AssignedRunner.Id)
+
 		require.NoError(runnerStream.Send(&pb.RunnerJobStreamRequest{
 			Event: &pb.RunnerJobStreamRequest_Ack_{
 				Ack: &pb.RunnerJobStreamRequest_Ack{},
@@ -449,14 +452,13 @@ func jobStreamRecv(
 
 func TestServiceQueueJob_odr(t *testing.T) {
 	require := require.New(t)
-
 	ctx := context.Background()
 
+	// Add a loud logger to our context
 	log := hclog.New(&hclog.LoggerOptions{
 		Name:  "odr-test",
 		Level: hclog.Trace,
 	})
-
 	ctx = hclog.WithContext(ctx, log)
 
 	// Create our server
@@ -464,7 +466,6 @@ func TestServiceQueueJob_odr(t *testing.T) {
 		WithLogger(log),
 		WithDB(testDB(t)),
 	)
-
 	require.NoError(err)
 	client := server.TestServer(t, impl, server.TestWithContext(ctx))
 
@@ -479,6 +480,7 @@ func TestServiceQueueJob_odr(t *testing.T) {
 	// Simplify writing tests
 	type Req = pb.QueueJobRequest
 
+	// Create an ODR profile
 	odr := serverptypes.TestOnDemandRunnerConfig(t, &pb.OnDemandRunnerConfig{
 		PluginType:   "magic-carpet",
 		PluginConfig: []byte("foo = 1"),
@@ -486,14 +488,11 @@ func TestServiceQueueJob_odr(t *testing.T) {
 			"CARPET_DRIVER": "apu",
 		},
 	})
-
 	cfgResp, err := client.UpsertOnDemandRunnerConfig(context.Background(), &pb.UpsertOnDemandRunnerConfigRequest{
 		Config: odr,
 	})
-
 	odr = cfgResp.Config
-
-	log.Info("test odr", "id", odr.Id)
+	log.Info("test odr profile", "id", odr.Id)
 
 	// Update the project to include ondemand runner
 	proj := serverptypes.TestProject(t, &pb.Project{
@@ -502,10 +501,10 @@ func TestServiceQueueJob_odr(t *testing.T) {
 			Id: odr.Id,
 		},
 	})
-
 	_, err = client.UpsertProject(context.Background(), &pb.UpsertProjectRequest{
 		Project: proj,
 	})
+	require.NoError(err)
 
 	// Create, should get an ID back
 	queueResp, err := client.QueueJob(ctx, &Req{
@@ -517,7 +516,6 @@ func TestServiceQueueJob_odr(t *testing.T) {
 		}),
 	})
 	require.NoError(err)
-	require.NotNil(queueResp)
 	require.NotEmpty(queueResp)
 
 	// Job should exist and be queued
@@ -539,14 +537,14 @@ func TestServiceQueueJob_odr(t *testing.T) {
 		},
 	}))
 
-	// Wait for assignment and ack
+	// We should get a task to start the job first.
 	resp, err := runnerStream.Recv()
 	require.NoError(err)
 	assignment, ok := resp.Event.(*pb.RunnerJobStreamResponse_Assignment)
 	require.True(ok, "should be an assignment")
 	require.NotNil(assignment)
+	require.Equal(id, assignment.Assignment.Job.AssignedRunner.Id)
 	require.NotEqual(queueResp.JobId, assignment.Assignment.Job.Id)
-
 	require.IsType(&pb.Job_StartTask{}, assignment.Assignment.Job.Operation)
 
 	st := assignment.Assignment.Job.Operation.(*pb.Job_StartTask).StartTask
@@ -557,8 +555,7 @@ func TestServiceQueueJob_odr(t *testing.T) {
 		require.Equal(v, st.Info.EnvironmentVariables[k])
 	}
 
-	runnerId := st.Info.EnvironmentVariables["WAYPOINT_RUNNER_ID"]
-
+	// Ack it and complete it
 	require.NoError(runnerStream.Send(&pb.RunnerJobStreamRequest{
 		Event: &pb.RunnerJobStreamRequest_Ack_{
 			Ack: &pb.RunnerJobStreamRequest_Ack{},
@@ -566,6 +563,7 @@ func TestServiceQueueJob_odr(t *testing.T) {
 	}))
 
 	// Register our runner
+	runnerId := st.Info.EnvironmentVariables["WAYPOINT_RUNNER_ID"]
 	TestRunner(t, client, &pb.Runner{
 		Id: runnerId,
 	})
@@ -581,6 +579,13 @@ func TestServiceQueueJob_odr(t *testing.T) {
 		},
 	}))
 
+	// Complete our launch task job so that we can move on
+	require.NoError(runnerStream.Send(&pb.RunnerJobStreamRequest{
+		Event: &pb.RunnerJobStreamRequest_Complete_{
+			Complete: &pb.RunnerJobStreamRequest_Complete{},
+		},
+	}))
+
 	// Wait for assignment and ack
 	resp, err = rs2.Recv()
 	require.NoError(err)
@@ -589,6 +594,7 @@ func TestServiceQueueJob_odr(t *testing.T) {
 	require.NotNil(assignment)
 	require.Equal(queueResp.JobId, assignment.Assignment.Job.Id)
 	require.Equal(runnerId, assignment.Assignment.Job.TargetRunner.Target.(*pb.Ref_Runner_Id).Id.Id)
+	require.Equal(runnerId, assignment.Assignment.Job.AssignedRunner.Id)
 
 	require.NoError(rs2.Send(&pb.RunnerJobStreamRequest{
 		Event: &pb.RunnerJobStreamRequest_Ack_{
@@ -726,6 +732,13 @@ func TestServiceQueueJob_odr_default(t *testing.T) {
 		},
 	}))
 
+	// Complete our launch task job so that we can move on
+	require.NoError(runnerStream.Send(&pb.RunnerJobStreamRequest{
+		Event: &pb.RunnerJobStreamRequest_Complete_{
+			Complete: &pb.RunnerJobStreamRequest_Complete{},
+		},
+	}))
+
 	// Register our runner
 	TestRunner(t, client, &pb.Runner{
 		Id: runnerId,
