@@ -13,6 +13,7 @@ import (
 	"github.com/hashicorp/go-memdb"
 	pb "github.com/hashicorp/waypoint/internal/server/gen"
 	serverptypes "github.com/hashicorp/waypoint/internal/server/ptypes"
+	"github.com/hashicorp/waypoint/internal/serverstate"
 )
 
 func TestAppOperation(t *testing.T) {
@@ -144,6 +145,13 @@ func TestAppOperation(t *testing.T) {
 		// to test our index is really doing the right thing.
 		rand.Shuffle(len(times), func(i, j int) { times[i], times[j] = times[j], times[i] })
 
+		// Since we search by latest sequence later, we don't want our latest
+		// to be sequence #1. This fixes a flaky test that would happen in
+		// that case cause we'd search for sequence 0 which doesn't exist.
+		if times[0] == latest {
+			times[0], times[1] = times[1], times[0]
+		}
+
 		// Create a build for each time
 		for _, timeVal := range times {
 			pt, err := ptypes.TimestampProto(timeVal)
@@ -185,7 +193,7 @@ func TestAppOperation(t *testing.T) {
 		require.Equal(b2.Sequence, b.Sequence-1)
 
 		// Try listing
-		builds, err := op.List(s, &listOperationsOptions{
+		builds, err := op.List(s, &serverstate.ListOperationOptions{
 			Application: ref,
 		})
 		require.NoError(err)
@@ -242,7 +250,7 @@ func TestAppOperation(t *testing.T) {
 		s := TestState(t)
 		defer s.Close()
 
-		_, err := op.List(s, &listOperationsOptions{})
+		_, err := op.List(s, &serverstate.ListOperationOptions{})
 		require.Error(err)
 	})
 
@@ -301,7 +309,7 @@ func TestAppOperation(t *testing.T) {
 		}
 
 		// List with a filter
-		results, err := op.List(s, &listOperationsOptions{
+		results, err := op.List(s, &serverstate.ListOperationOptions{
 			Application: ref,
 			Status: []*pb.StatusFilter{
 				{
@@ -352,7 +360,7 @@ func TestAppOperation(t *testing.T) {
 
 		// List with a filter
 		build := serverptypes.TestValidBuild(t, nil)
-		results, err := op.List(s, &listOperationsOptions{
+		results, err := op.List(s, &serverstate.ListOperationOptions{
 			Application: build.Application,
 			Workspace:   &pb.Ref_Workspace{Workspace: "WS_A"},
 		})
@@ -401,7 +409,7 @@ func TestAppOperation(t *testing.T) {
 
 		// List with a filter
 		build := serverptypes.TestValidBuild(t, nil)
-		results, err := op.List(s, &listOperationsOptions{
+		results, err := op.List(s, &serverstate.ListOperationOptions{
 			Application: build.Application,
 		})
 		require.NoError(err)
@@ -431,7 +439,7 @@ func TestAppOperation(t *testing.T) {
 
 		// List with a filter
 		build := serverptypes.TestValidBuild(t, nil)
-		results, err := op.List(s, &listOperationsOptions{
+		results, err := op.List(s, &serverstate.ListOperationOptions{
 			Application:   build.Application,
 			PhysicalState: pb.Operation_CREATED,
 		})
@@ -485,7 +493,7 @@ func TestAppOperation(t *testing.T) {
 		require.Equal(0, len(ws))
 
 		// List with a filter
-		results, err := op.List(s, &listOperationsOptions{
+		results, err := op.List(s, &serverstate.ListOperationOptions{
 			Application: ref,
 			WatchSet:    ws,
 		})
@@ -646,7 +654,7 @@ func TestAppOperation_deploy(t *testing.T) {
 
 		// List with a filter
 		deploy := serverptypes.TestValidDeployment(t, nil)
-		results, err := op.List(s, &listOperationsOptions{
+		results, err := op.List(s, &serverstate.ListOperationOptions{
 			Application:   deploy.Application,
 			PhysicalState: pb.Operation_CREATED,
 		})
