@@ -1,9 +1,3 @@
-import Service from '@ember/service';
-import { DEBUG } from '@glimmer/env';
-import { WaypointClient } from 'waypoint-client';
-import SessionService from 'waypoint/services/session';
-import { inject as service } from '@ember/service';
-import { buildWaiter } from '@ember/test-waiters';
 import {
   Application,
   Build,
@@ -13,6 +7,7 @@ import {
   ListBuildsRequest,
   ListBuildsResponse,
   ListPushedArtifactsRequest,
+  ListWorkspacesRequest,
   OperationOrder,
   Project,
   PushedArtifact,
@@ -20,14 +15,22 @@ import {
   Release,
   StatusFilter,
   StatusReport,
+  UI,
   UpsertProjectRequest,
   Variable,
-  UI,
+  Workspace,
 } from 'waypoint-pb';
-import { Request, Metadata, UnaryResponse, UnaryInterceptor } from 'grpc-web';
-import { Message } from 'google-protobuf';
+import { Metadata, Request, UnaryInterceptor, UnaryResponse } from 'grpc-web';
+
+import { DEBUG } from '@glimmer/env';
 import { Empty } from 'google-protobuf/google/protobuf/empty_pb';
+import { Message } from 'google-protobuf';
+import Service from '@ember/service';
+import SessionService from 'ember-simple-auth/services/session';
+import { WaypointClient } from 'waypoint-client';
+import { buildWaiter } from '@ember/test-waiters';
 import config from 'waypoint/config/environment';
+import { inject as service } from '@ember/service';
 
 // The docs for @ember/test-waiters recommend building waiters in module scope.
 // https://github.com/emberjs/ember-test-waiters#use-buildwaiter-in-module-scope
@@ -43,6 +46,11 @@ const protocolVersions = {
   'client-version': 'ui-0.0.1',
 };
 
+export type BuildExtended = Build.AsObject & {
+  statusReport?: StatusReport.AsObject;
+  pushedArtifact?: PushedArtifact.AsObject;
+};
+
 // This is an adapter type. It exists to let us transition to
 // UI.DeploymentBundle incrementally.
 export type DeploymentExtended = Deployment.AsObject & {
@@ -82,8 +90,8 @@ export default class ApiService extends Service {
   }
 
   get meta(): Metadata {
-    if (this.session.authConfigured) {
-      return { ...protocolVersions, authorization: this.session.token };
+    if (this.session.isAuthenticated) {
+      return { ...protocolVersions, authorization: this.session.data.authenticated?.token as string };
     } else {
       return { ...protocolVersions };
     }
@@ -301,12 +309,16 @@ export default class ApiService extends Service {
         git.clearBasic();
         git.clearSsh();
       }
+
+      ref.setRemoteEnabled(true);
     } else {
       // if we set up a project without connecting it to a git repo
       // but we want to set input variables, a git URL is required
       // for updating a project's settings. this silences that error
       // while not adding settings the user did not specify
       git.setUrl('\n');
+
+      ref.setRemoteEnabled(project.remoteEnabled);
     }
 
     dataSource.setGit(git);
@@ -339,6 +351,18 @@ export default class ApiService extends Service {
     let respProject = resp.toObject().project;
     return respProject;
   }
+
+  async listWorkspaces(projectRef?: Ref.Project): Promise<Workspace.AsObject[]> {
+    let req = new ListWorkspacesRequest();
+
+    if (projectRef) {
+      req.setProject(projectRef);
+    }
+
+    let resp = await this.client.listWorkspaces(req, this.WithMeta());
+
+    return resp.toObject().workspacesList;
+  }
 }
 
 // DO NOT DELETE: this is how TypeScript knows how to look up your services.
