@@ -1,34 +1,82 @@
-import { module, test } from 'qunit';
+import { clickable, collection, create, visitable } from 'ember-cli-page-object';
 import { currentURL, findAll } from '@ember/test-helpers';
+import { module, test } from 'qunit';
+
 import { setupApplicationTest } from 'ember-qunit';
 import { setupMirage } from 'ember-cli-mirage/test-support';
-import { visitable, create, collection, clickable } from 'ember-cli-page-object';
-import login from 'waypoint/tests/helpers/login';
+import { setupSession } from 'waypoint/tests/helpers/login';
 
 const url = '/default/microchip/app/wp-bandwidth/deployments';
+const emptyStateUrl = '/default/microchip/app/wp-bandwidth/deployment';
+const redirectUrl = '/default/microchip/app/wp-bandwidth/deployment/seq/'; // concat with length of deployments array in test
 
 const page = create({
   visit: visitable(url),
-  list: collection('[data-test-deployment-list] li'),
-  deploymentLinks: collection('[data-test-external-deployment-button]'),
-  destroyedBadges: collection('[data-test-destroyed-badge]'),
   showDestroyed: clickable('[data-test-display-destroyed-button]'),
+  linkList: collection('[data-test-deployment-list-item]'),
+  aliveStatusIndicators: collection('[data-test-health-status="alive"]'),
+  healthCheckOrDeployUrls: collection('.health-check--text-description'),
+  operationStatuses: collection('[data-test-operation-status]'),
+  gitCommits: collection('[data-test-git-commit]'),
 });
 
 module('Acceptance | deployments list', function (hooks) {
   setupApplicationTest(hooks);
   setupMirage(hooks);
-  login();
+  setupSession(hooks);
 
-  test('visiting deployments page', async function (assert) {
+  test('happy path', async function (assert) {
+    let project = this.server.create('project', { name: 'microchip' });
+    let application = this.server.create('application', { name: 'wp-bandwidth', project });
+    let deployments = this.server.createList('deployment', 3, 'random', { application });
+    this.server.create('status-report', 'alive', { application, target: deployments[0] });
+    this.server.create('status-report', 'ready', { application, target: deployments[1] });
+
+    await page.visit();
+
+    assert.equal(page.linkList.length, 3);
+    assert.equal(currentURL(), redirectUrl + '3');
+    assert.equal(page.aliveStatusIndicators.length, 1);
+    assert.equal(page.operationStatuses.length, 3);
+    assert.equal(page.gitCommits.length, 3);
+    assert.equal(page.healthCheckOrDeployUrls[1].text, 'Starting…');
+    assert.equal(page.healthCheckOrDeployUrls[0].text, 'wildly-intent-honeybee--v2.waypoint.run');
+  });
+
+  test('visiting deployments page redirects to latest', async function (assert) {
     let project = this.server.create('project', { name: 'microchip' });
     let application = this.server.create('application', { name: 'wp-bandwidth', project });
     this.server.createList('deployment', 3, 'random', { application });
 
     await page.visit();
 
-    assert.equal(page.list.length, 3);
-    assert.equal(currentURL(), url);
+    assert.equal(page.linkList.length, 3);
+    assert.equal(currentURL(), redirectUrl + '3');
+  });
+
+  test('empty deployments list provides empty state ui', async function (assert) {
+    let project = this.server.create('project', { name: 'microchip' });
+    this.server.create('application', { name: 'wp-bandwidth', project });
+
+    await page.visit();
+
+    assert.equal(page.linkList.length, 0);
+    assert.equal(currentURL(), emptyStateUrl);
+    assert.dom('.empty-state').exists();
+  });
+
+  test('clicking a different deployment moves us to that details page', async function (assert) {
+    let project = this.server.create('project', { name: 'microchip' });
+    let application = this.server.create('application', { name: 'wp-bandwidth', project });
+    this.server.createList('deployment', 3, 'random', { application });
+
+    await page.visit();
+    await page.linkList[1].click();
+    let elems = findAll('[data-test-deployment-list] li');
+
+    assert.dom(elems[0]).doesNotHaveClass('active');
+    assert.dom(elems[1]).hasClass('active');
+    assert.equal(currentURL(), redirectUrl + 2);
   });
 
   test('visiting deployments page with mutable deployments', async function (assert) {
@@ -75,41 +123,10 @@ module('Acceptance | deployments list', function (hooks) {
 
     await page.visit();
 
-    assert.equal(page.list.length, 4);
-    assert.equal(page.deploymentLinks.length, 1);
+    assert.equal(page.linkList.length, 4);
 
     await page.showDestroyed();
 
-    assert.equal(page.list.length, 5);
-    assert.equal(page.deploymentLinks.length, 1);
-    assert.equal(page.destroyedBadges.length, 1);
-  });
-
-  test('status reports appear where available', async function (assert) {
-    let project = this.server.create('project', { name: 'microchip' });
-    let application = this.server.create('application', { name: 'wp-bandwidth', project });
-
-    this.server.create('deployment', 'random', {
-      application,
-      sequence: 3,
-      statusReport: this.server.create('status-report', 'alive', { application }),
-    });
-    this.server.create('deployment', 'random', {
-      application,
-      sequence: 2,
-      statusReport: this.server.create('status-report', 'ready', { application }),
-    });
-    this.server.create('deployment', 'random', {
-      application,
-      sequence: 1,
-      statusReport: this.server.create('status-report', 'down', { application }),
-    });
-
-    await page.visit();
-
-    let badges = findAll(`[data-test-deployment-list] [data-test-status-report-indicator]`);
-    let statuses = badges.map((b) => b.getAttribute('data-test-status-report-indicator'));
-
-    assert.deepEqual(statuses, ['alive', 'ready', 'down'], `correct status badges appear in deployment-list`);
+    assert.equal(page.linkList.length, 5);
   });
 });
