@@ -1,14 +1,15 @@
 import { module, test } from 'qunit';
-import { visit } from '@ember/test-helpers';
+
+import percySnapshot from '@percy/ember';
 import { setupApplicationTest } from 'ember-qunit';
 import { setupMirage } from 'ember-cli-mirage/test-support';
-import percySnapshot from '@percy/ember';
-import login from '../helpers/login';
+import { setupSession } from 'waypoint/tests/helpers/login';
+import { visit } from '@ember/test-helpers';
 
 module('Acceptance | Percy', function (hooks) {
   setupApplicationTest(hooks);
   setupMirage(hooks);
-  login();
+  setupSession(hooks);
 
   test('empty projects list', async function (assert) {
     await visit('/default');
