@@ -1,5 +1,16 @@
 import { ContextObject, RunOptions } from 'axe-core';
-import { setupGlobalA11yHooks, setEnableA11yAudit, setRunOptions } from 'ember-a11y-testing/test-support';
+import {
+  setupGlobalA11yHooks,
+  setEnableA11yAudit,
+  setRunOptions,
+  DEFAULT_A11Y_TEST_HELPER_NAMES,
+} from 'ember-a11y-testing/test-support';
+
+const A11Y_TEST_HELPER_NAMES: typeof DEFAULT_A11Y_TEST_HELPER_NAMES = [
+  ...DEFAULT_A11Y_TEST_HELPER_NAMES,
+  'render',
+  'focus',
+];
 
 // ember-a11y-testing allows us to pass `include` and `exclude` context
 // parameters as run options. This isn’t documented, and isn’t represented in
@@ -16,7 +27,7 @@ const include = [['#ember-testing-container']];
 const exclude = [['.pds-logomark'], ['.pds-tabNav'], ['.card-header'], ['.x-toggle-btn']];
 
 export function setup(): void {
-  setupGlobalA11yHooks(() => true);
+  setupGlobalA11yHooks(() => true, { helpers: A11Y_TEST_HELPER_NAMES });
   setEnableA11yAudit(true);
   setRunOptions({ include, exclude } as OptionsWithContext);
 }
